@@ -30,12 +30,17 @@ from zope import component
 from collection import collection
 from event import AttributeChangeEvent, AssociationSetEvent, \
                   AssociationAddEvent, AssociationDeleteEvent
-from event import DerivedUnionSetEvent, DerivedUnionAddEvent, \
-                  DerivedUnionDeleteEvent
+from event import DerivedChangeEvent, DerivedSetEvent, \
+                  DerivedAddEvent, DerivedDeleteEvent
 from event import RedefineSetEvent, RedefineAddEvent, RedefineDeleteEvent
-from interfaces import IAssociationChangeEvent, IAssociationSetEvent, \
+from interfaces import IElementChangeEvent, \
+                       IAssociationChangeEvent, IAssociationSetEvent, \
                        IAssociationAddEvent, IAssociationDeleteEvent
 import operator
+import time
+
+# Stamping version time on derivedunions
+stamp = time.clock
 
 
 class umlproperty(object):
@@ -147,11 +152,16 @@ class enumeration(umlproperty):
     predefined set of values. Multiplicity is always `[0..1]`
     """
 
+    # All enumerations have a type 'str'
+    type = property(lambda s: str)
+
     def __init__(self, name, values, default):
         self.name = intern(name)
         self._name = intern('_' + name)
         self.values = values
         self.default = default
+        self.lower = 0
+        self.upper = 1
 
     def __str__(self):
         return '<enumeration %s: %s = %s>' % (self.name, self.values, self.default)
@@ -303,12 +313,6 @@ class association(umlproperty):
             if do_notify:
                 event = AssociationAddEvent(obj, self, value)
 
-        # Callbacks are only connected if a new relationship has
-        # been established.
-#        value.connect('__unlink__', self.__on_unlink, obj)
-#        if self.composite:
-#            obj.connect('__unlink__', self.__on_composite_unlink, value)
-        
         if not from_opposite and self.opposite:
             opposite = getattr(type(value), self.opposite)
             if not opposite.opposite:
@@ -368,9 +372,6 @@ class association(umlproperty):
                 if do_notify:
                     event = AssociationSetEvent(obj, self, value, None)
 
-#        value.disconnect(self.__on_unlink, obj)
-#        if self.composite:
-#            obj.disconnect(self.__on_composite_unlink, value)
         if do_notify and event:
             component.handle(event)
 
@@ -437,21 +438,40 @@ class associationstub(umlproperty):
             setattr(obj, self._name, set([value]))
             
     def _del(self, obj, value, from_opposite=False):
-        getattr(obj, self._name).discard(value)
+        try:
+            c = getattr(obj, self._name)
+        except AttributeError:
+            pass
+        else:
+            c.discard(value)
 
 
-class derivedunion(umlproperty):
+class unioncache(object):
     """
-    Derived union
-
-      Element.union = derivedunion('union', subset1, subset2..subsetn)
-
-    The subsets are the properties that participate in the union (Element.name).
+    Small cache helper object for derivedunions.
     """
 
-    def __init__(self, name, lower, upper, *subsets):
+    def __init__(self, data, version):
+        self.data = data
+        self.version = version
+
+
+class derived(umlproperty):
+    """
+    Base class for derived properties, both derived unions and custom
+    properties.
+
+    Note that, although this derived property sends DerivedAddEvent,
+    -Delete- and Set events, this gives just an assumption that something
+    may have changed. If something actually changed depends on the filter
+    applied to the derived property.
+    """
+
+    def __init__(self, name, type, lower, upper, *subsets):
         self.name = intern(name)
         self._name = intern('_' + name)
+        self.version = 1
+        self.type = type
         self.lower = lower
         self.upper = upper
         self.subsets = set(subsets)
@@ -464,12 +484,117 @@ class derivedunion(umlproperty):
         raise ValueError, 'Derivedunion: Properties should not be loaded in a derived union %s: %s' % (self.name, value)
 
 
+    def postload(self, obj):
+        self.version += 1
+
     def save(self, obj, save_func):
         pass
 
 
     def __str__(self):
-        return '<derivedunion %s: %s>' % (self.name, str(map(str, self.subsets))[1:-1])
+        return '<derived %s: %s>' % (self.name, str(map(str, self.subsets))[1:-1])
+
+    def filter(self, obj):
+        """
+        Filter should return something iterable.
+        """
+        raise NotImplementedError, 'Implement this in the property.'
+
+    def _update(self, obj):
+        """
+        Update the list of items. Returns a unioncache instance.
+        """
+        u = self.filter(obj)
+        if self.upper <= 1:
+            assert len(u) <= 1, 'Derived union %s of item %s should have length 1 %s' % (self.name, obj.id, tuple(u))
+            # maybe code below is better instead the assertion above?
+            #if len(u) > 1:
+            #    log.warning('Derived union %s of item %s should have length 1 %s' % (self.name, obj.id, tuple(u)))
+            if u:
+                u = iter(u).next()
+            else:
+                u = None
+
+        uc = unioncache(u, self.version)
+        setattr(obj, self._name, uc)
+        return uc
+        
+        
+    def _get(self, obj):
+        try:
+            uc = getattr(obj, self._name)
+            if uc.version != self.version:
+                uc = self._update(obj)
+        except AttributeError:
+            uc = self._update(obj)
+        
+        return uc.data
+
+
+    def _set(self, obj, value):
+        raise AttributeError, 'Can not set values on a union'
+
+
+    def _del(self, obj, value=None):
+        raise AttributeError, 'Can not delete values on a union'
+
+    @component.adapter(IElementChangeEvent)
+    def _association_changed(self, event):
+        """
+        Re-emit state change for the derived properties as Derived*Event's.
+
+        TODO: We should fetch the old and new state of the namespace item in
+        stead of the old and new values of the item that changed.
+
+        If multiplicity is [0..1]:
+          send DerivedSetEvent if len(union) < 2
+        if multiplicity is [*]:
+          send DerivedAddEvent and DerivedDeleteEvent
+            if value not in derived union and 
+        """
+        if event.property in self.subsets:
+            # Make sure unions are created again
+            self.version += 1
+            
+            if not IAssociationChangeEvent.providedBy(event):
+                return
+                
+            # mimic the events for Set/Add/Delete
+            if self.upper > 1:
+                if IAssociationSetEvent.providedBy(event):
+                    old_value, new_value = event.old_value, event.new_value
+                    # Do a filter? Change to 
+                    component.handle(DerivedDeleteEvent(event.element, self, old_value))
+                    component.handle(DerivedAddEvent(event.element, self, new_value))
+
+                elif IAssociationAddEvent.providedBy(event):
+                    new_value = event.new_value
+                    component.handle(DerivedAddEvent(event.element, self, new_value))
+
+                elif IAssociationDeleteEvent.providedBy(event):
+                    old_value = event.old_value
+                    component.handle(DerivedDeleteEvent(event.element, self, old_value))
+
+                elif IAssociationChangeEvent.providedBy(event):
+                    component.handle(DerivedChangeEvent(event.element, self))
+                else:
+                    log.error('Don''t know how to handle event ' + str(event) + ' for derived union')
+            else:        
+                # This is a [0..1] event
+                # TODO: This is an error: [0..*] associations may be used for updating [0..1] associations
+                assert IAssociationSetEvent.providedBy(event)
+                old_value, new_value = event.old_value, event.new_value
+                component.handle(DerivedSetEvent(event.element, self, old_value, new_value))
+
+
+class derivedunion(derived):
+    """
+    Derived union
+
+      Element.union = derivedunion('union', subset1, subset2..subsetn)
+
+    The subsets are the properties that participate in the union (Element.name).
+    """
 
     def _union(self, obj, exclude=None):
         """
@@ -491,56 +616,52 @@ class derivedunion(umlproperty):
                         u.add(tmp)
             return u
 
-    def _get(self, obj):
-        u = self._union(obj)
-        if self.upper > 1:
-            return u
-        else:
-            assert len(u) <= 1, 'Derived union %s of item %s should have length 1 %s' % (self.name, obj.id, tuple(u))
-            return u and iter(u).next() or None
-
-
-    def _set(self, obj, value):
-        raise AttributeError, 'Can not set values on a union'
-
-
-    def _del(self, obj, value=None):
-        raise AttributeError, 'Can not delete values on a union'
-
-
-    @component.adapter(IAssociationChangeEvent)
+    # Filter is our default filter
+    filter = _union
+    
+    @component.adapter(IElementChangeEvent)
     def _association_changed(self, event):
         """
-        Re-emit state change for the derived union (as DerivedUnion*Event's).
+        Re-emit state change for the derived union (as Derived*Event's).
 
         TODO: We should fetch the old and new state of the namespace item in
         stead of the old and new values of the item that changed.
 
         If multiplicity is [0..1]:
-          send DerivedUnionSetEvent if len(union) < 2
+          send DerivedSetEvent if len(union) < 2
         if multiplicity is [*]:
-          send DerivedUnionAddEvent and DerivedUnionDeleteEvent
+          send DerivedAddEvent and DerivedDeleteEvent
             if value not in derived union and 
         """
         if event.property in self.subsets:
-            # mimic the events for Set/Add/Delete
+            # Make sure unions are created again
+            self.version += 1
+            
+            if not IAssociationChangeEvent.providedBy(event):
+                return
+                
+            values = self._union(event.element, exclude=event.property)
 
             if self.upper > 1:
                 if IAssociationSetEvent.providedBy(event):
                     old_value, new_value = event.old_value, event.new_value
-                    if old_value and old_value not in self._union(event.element, exclude=event.property):
-                        component.handle(DerivedUnionDeleteEvent(event.element, self, old_value))
-                    if new_value and new_value not in self._union(event.element, exclude=event.property):
-                        component.handle(DerivedUnionAddEvent(event.element, self, new_value))
+                    if old_value and old_value not in values:
+                        component.handle(DerivedDeleteEvent(event.element, self, old_value))
+                    if new_value and new_value not in values:
+                        component.handle(DerivedAddEvent(event.element, self, new_value))
 
                 elif IAssociationAddEvent.providedBy(event):
                     new_value = event.new_value
-                    if new_value not in self._union(event.element, exclude=event.property):
-                        component.handle(DerivedUnionAddEvent(event.element, self, new_value))
+                    if new_value not in values:
+                        component.handle(DerivedAddEvent(event.element, self, new_value))
+
                 elif IAssociationDeleteEvent.providedBy(event):
                     old_value = event.old_value
-                    if old_value not in self._union(event.element, exclude=event.property):
-                        component.handle(DerivedUnionDeleteEvent(event.element, self, old_value))
+                    if old_value not in values:
+                        component.handle(DerivedDeleteEvent(event.element, self, old_value))
+
+                elif IAssociationChangeEvent.providedBy(event):
+                    component.handle(DerivedChangeEvent(event.element, self))
                 else:
                     log.error('Don''t know how to handle event ' + str(event) + ' for derived union')
             else:        
@@ -549,10 +670,8 @@ class derivedunion(umlproperty):
                 # This is a [0..1] event
                 if self.single:
                     # Only one subset element, so pass the values on
-                    component.handle(DerivedUnionSetEvent(event.element, self, old_value, new_value))
+                    component.handle(DerivedSetEvent(event.element, self, old_value, new_value))
                 else:
-                    values = self._union(event.element, exclude=event.property)
-
                     new_values = set(values)
                     if new_value:
                         new_values.add(new_value)
@@ -561,7 +680,7 @@ class derivedunion(umlproperty):
                         return
                     if values:
                         new_value = iter(values).next()
-                    component.handle(DerivedUnionSetEvent(event.element, self, old_value, new_value))
+                    component.handle(DerivedSetEvent(event.element, self, old_value, new_value))
 
 
 class redefine(umlproperty):
@@ -592,11 +711,21 @@ class redefine(umlproperty):
             self.original.load(obj, value)
 
 
+    def postload(self, obj):
+        if self.original.name == self.name:
+            self.original.postload(obj)
+
+
     def save(self, obj, save_func):
         if self.original.name == self.name:
             self.original.save(obj, save_func)
 
 
+    def unlink(self, obj):
+        if self.original.name == self.name:
+            self.original.unlink(obj)
+
+
     def __str__(self):
         return '<redefine %s: %s = %s>' % (self.name, self.type.__name__, str(self.original))
 
