@@ -9,10 +9,28 @@ from gaphas import geometry
 from gaphas import constraint
 from gaphor import UML
 from gaphor.core import inject
-from gaphor.UML.umllex import render_attribute
 from gaphor.diagram.interfaces import IEditor
 from gaphor.diagram import items
 from gaphor.misc.rattr import rgetattr, rsetattr
+from simplegeneric import generic
+
+
+@generic
+def editable(el):
+    """
+    Return editable part of UML element.
+
+    It returns element itself by default.
+    """
+    return el
+    
+
+@editable.when_type(UML.Slot)
+def editable_slot(el):
+    """
+    Return editable part of a slot.
+    """
+    return el.value
 
 
 class CommentItemEditor(object):
@@ -57,13 +75,15 @@ class NamedItemEditor(object):
         return True
 
     def get_text(self):
-        return self._item.subject.name
+        s = self._item.subject
+        return s.name if s else ''
 
     def get_bounds(self):
         return None
 
     def update_text(self, text):
-        self._item.subject.name = text
+        if self._item.subject:
+            self._item.subject.name = text
         self._item.request_update()
 
     def key_pressed(self, pos, key):
@@ -101,6 +121,7 @@ class DiagramItemTextEditor(object):
         return None
 
     def update_text(self, text):
+        log.debug('Updating text to %s' % text)
         if self._text_element:
             self._text_element.text = text
             rsetattr(self._item.subject, self._text_element.attr, text)
@@ -111,13 +132,12 @@ class DiagramItemTextEditor(object):
 component.provideAdapter(DiagramItemTextEditor)
 
 
-class ClassifierItemEditor(object):
+class CompartmentItemEditor(object):
     """
-    Text editor support for Classifiers. Also features contained in Classifiers'
-    compartments are edited through this interface.
+    Text editor support for compartment items.
     """
     interface.implements(IEditor)
-    component.adapts(items.ClassifierItem)
+    component.adapts(items.CompartmentItem)
 
     def __init__(self, item):
         self._item = item
@@ -132,23 +152,18 @@ class ClassifierItemEditor(object):
         return bool(self._edit and self._edit.subject)
 
     def get_text(self):
-        if hasattr(self._edit.subject, 'render'):
-            return self._edit.subject.render()
-        return self._edit.subject.name
+        return UML.format(editable(self._edit.subject))
 
     def get_bounds(self):
         return None
 
     def update_text(self, text):
-        if hasattr(self._edit.subject, 'parse'):
-            return self._edit.subject.parse(text)
-        else:
-            self._item.subject.name = text
+        UML.parse(editable(self._edit.subject), text)
 
     def key_pressed(self, pos, key):
         pass
 
-component.provideAdapter(ClassifierItemEditor)
+component.provideAdapter(CompartmentItemEditor)
  
 
 class AssociationItemEditor(object):
@@ -166,9 +181,9 @@ class AssociationItemEditor(object):
         item = self._item
         if not item.subject:
             return False
-        if item.head_end.point(x, y) <= 0:
+        if item.head_end.point((x, y)) <= 0:
             self._edit = item.head_end
-        elif item.tail_end.point(x, y) <= 0:
+        elif item.tail_end.point((x, y)) <= 0:
             self._edit = item.tail_end
         else:
             self._edit = item
@@ -177,16 +192,14 @@ class AssociationItemEditor(object):
     def get_text(self):
         if self._edit is self._item:
             return self._edit.subject.name
-        return render_attribute(self._edit.subject)
-
+        return UML.format(self._edit.subject, visibility=True,
+                                is_derived=True, type=True,
+                                multiplicity=True, default=True)
     def get_bounds(self):
         return None
 
     def update_text(self, text):
-        if hasattr(self._edit.subject, 'parse'):
-            return self._edit.subject.parse(text)
-        else:
-            self._item.subject.name = text
+        UML.parse(self._edit.subject, text)
 
     def key_pressed(self, pos, key):
         pass
