@@ -9,12 +9,20 @@ All important services are present in the application object:
  - action sets
 """
 
+import os
 import pkg_resources
 from zope import component
 from gaphor.interfaces import IService, IEventFilter
 from gaphor.event import ServiceInitializedEvent, ServiceShutdownEvent
 import gaphor.UML
 
+if os.name == 'nt':
+    home = 'USERPROFILE'
+else:
+    home = 'HOME'
+
+user_data_dir = os.path.join(os.getenv(home), '.gaphor')
+
 
 class _Application(object):
     """
