@@ -6,7 +6,7 @@ import math
 
 from gaphas.util import path_ellipse
 from gaphas.state import observed, reversible_property
-from gaphas.item import Handle, Item
+from gaphas.item import Handle, Item, LinePort
 from gaphas.constraint import EqualsConstraint, LessThanConstraint
 from gaphas.geometry import distance_line_point
 
@@ -205,7 +205,10 @@ class ForkNodeItem(Item, DiagramItem):
         Item.__init__(self)
         DiagramItem.__init__(self, id)
         
-        self._handles.extend((Handle(), Handle()))
+        h1, h2 = Handle(), Handle()
+        self._handles.append(h1)
+        self._handles.append(h2)
+        self._ports.append(LinePort(h1.pos, h2.pos))
 
         self._combined = None
 
@@ -222,14 +225,13 @@ class ForkNodeItem(Item, DiagramItem):
                     'text-align-group': 'stereotype',
                 }, editable=True)
 
-        self.add_watch(UML.NamedElement.name, self.on_named_element_name)
-        self.add_watch(UML.JoinNode.joinSpec, self.on_join_node_join_spec)
-        self.add_watch(UML.LiteralSpecification.value, self.on_join_node_join_spec)
+        self.watch('subject<NamedElement>.name', self.on_named_element_name)\
+            .watch('subject<JoinNode>.joinSpec<LiteralSpecification>.value', self.on_join_node_join_spec)
 
 
     def save(self, save_func):
         save_func('matrix', tuple(self.matrix))
-        save_func('height', float(self._handles[1].y))
+        save_func('height', float(self._handles[1].pos.y))
         if self._combined:
             save_func('combined', self._combined, reference=True)
         DiagramItem.save(self, save_func)
@@ -238,7 +240,7 @@ class ForkNodeItem(Item, DiagramItem):
         if name == 'matrix':
             self.matrix = eval(value)
         elif name == 'height':
-            self._handles[1].y = eval(value)
+            self._handles[1].pos.y = eval(value)
         elif name == 'combined':
             self._combined = value
         else:
@@ -249,6 +251,7 @@ class ForkNodeItem(Item, DiagramItem):
         subject = self.subject
         if subject and isinstance(subject, UML.JoinNode) and subject.joinSpec:
             self._join_spec.text = self.subject.joinSpec.value
+        self.on_named_element_name(None)
         super(ForkNodeItem, self).postload()
 
     @observed
@@ -263,8 +266,8 @@ class ForkNodeItem(Item, DiagramItem):
 
         h1, h2 = self._handles
         cadd = self.canvas.solver.add_constraint
-        c1 = EqualsConstraint(a=h1.x, b=h2.x)
-        c2 = LessThanConstraint(smaller=h1.y, bigger=h2.y, delta=30)
+        c1 = EqualsConstraint(a=h1.pos.x, b=h2.pos.x)
+        c2 = LessThanConstraint(smaller=h1.pos.y, bigger=h2.pos.y, delta=30)
         self._constraints.extend((cadd(c1), cadd(c2)))
 
 
@@ -284,7 +287,7 @@ class ForkNodeItem(Item, DiagramItem):
     def text_align(self, extents, align, padding, outside):
         h1, h2 = self._handles
         w, _ = self.style.min_size
-        h = h2.y - h1.y
+        h = h2.pos.y - h1.pos.y
         x, y = get_text_point(extents, w, h, align, padding, outside)
 
         return x, y
@@ -313,34 +316,33 @@ class ForkNodeItem(Item, DiagramItem):
 
         cr.set_line_width(6)
         h1, h2 = self._handles
-        cr.move_to(h1.x, h1.y)
-        cr.line_to(h2.x, h2.y)
+        cr.move_to(h1.pos.x, h1.pos.y)
+        cr.line_to(h2.pos.x, h2.pos.y)
 
         cr.stroke()
 
 
-    def point(self, x, y):
+    def point(self, pos):
         h1, h2 = self._handles
-        d, p = distance_line_point(h1.pos, h2.pos, (x, y))
+        d, p = distance_line_point(h1.pos, h2.pos, pos)
         # Substract line_width / 2
         return d - 3
 
 
     def on_named_element_name(self, event):
+        print 'on_named_element_name', self.subject
         subject = self.subject
-        if subject and (event is None or event.element is subject):
+        if subject:
             self._name.text = subject.name
             self.request_update()
 
     def on_join_node_join_spec(self, event):
         subject = self.subject
-        if subject and is_join_node(subject) and \
-                (event is None or \
-                 event.element is subject.joinSpec or \
-                 event.element is subject.joinSpec.value):
-            if is_join_node(subject) and not (subject.joinSpec and subject.joinSpec.value):
-                self.set_join_spec(DEFAULT_JOIN_SPEC)
-            self.request_update()
+        try:
+            self._join_spec.text = subject.joinSpec.value
+        except AttributeError, e:
+            self.set_join_spec(DEFAULT_JOIN_SPEC)
+        self.request_update()
 
 
     def set_join_spec(self, value):
@@ -358,7 +360,6 @@ class ForkNodeItem(Item, DiagramItem):
             value = DEFAULT_JOIN_SPEC
 
         subject.joinSpec.value = value
-        self._join_spec.text = value
 
 
 def is_join_node(subject):
