@@ -43,18 +43,22 @@ class AssociationItem(NamedLine):
         self._dir_angle = 0
         self._dir_pos = 0, 0
         
-        self.add_watch(UML.Association.ownedEnd)
-        self.add_watch(UML.Association.memberEnd)
+        #self.watch('subject<Association>.ownedEnd')\
+            #.watch('subject<Association>.memberEnd')
 
         # For the association ends:
-        self.add_watch(UML.Property.aggregation, self.on_association_end_value)
-        self.add_watch(UML.Property.owningAssociation, self.on_association_end_value)
-        self.add_watch(UML.Property.classifier, self.on_association_end_value)
-        self.add_watch(UML.Property.visibility, self.on_association_end_value)
-        #self.add_watch(UML.Property.name, self.on_association_end_value)
-        # lowerValue, upperValue and taggedValue
-        self.add_watch(UML.LiteralSpecification.value, self.on_association_end_value)
-
+        base = 'subject<Association>.memberEnd<Property>.'
+        self.watch(base + 'name', self.on_association_end_value)\
+            .watch(base + 'aggregation', self.on_association_end_value)\
+            .watch(base + 'classifier', self.on_association_end_value)\
+            .watch(base + 'visibility', self.on_association_end_value)\
+            .watch(base + 'lowerValue<LiteralSpecification>.value', self.on_association_end_value)\
+            .watch(base + 'upperValue<LiteralSpecification>.value', self.on_association_end_value)\
+            .watch(base + 'owningAssociation', self.on_association_end_value) \
+            .watch(base + 'type<Class>.ownedAttribute', self.on_association_end_value) \
+            .watch(base + 'type<Interface>.ownedAttribute', self.on_association_end_value) \
+            .watch('subject<Association>.ownedEnd') \
+            .watch('subject<Association>.navigableOwnedEnd')
 
     def set_show_direction(self, dir):
         self._show_direction = dir
@@ -132,19 +136,19 @@ class AssociationItem(NamedLine):
         """
         Handle events and update text on association end.
         """
-        if event:
-            element = event.element
-            for end in (self._head_end, self._tail_end):
-                subject = end.subject
-                if subject and element in (subject, subject.lowerValue, \
-                        subject.upperValue, subject.taggedValue):
-                    end.set_text()
-                    self.request_update()
-                    break;
-        else:
-            for end in (self._head_end, self._tail_end):
-                end.set_text()
-            self.request_update()
+        #if event:
+        #    element = event.element
+        #    for end in (self._head_end, self._tail_end):
+        #        subject = end.subject
+        #        if subject and element in (subject, subject.lowerValue, \
+        #                subject.upperValue, subject.taggedValue):
+        #            end.set_text()
+        #            self.request_update()
+        ##            break;
+        #else:
+        for end in (self._head_end, self._tail_end):
+            end.set_text()
+        self.request_update()
 
             
 
@@ -165,9 +169,9 @@ class AssociationItem(NamedLine):
                 self.draw_head = self.draw_head_composite
             elif tail_subject.aggregation == intern('shared'):
                 self.draw_head = self.draw_head_shared
-            elif self._head_end.navigability:
+            elif self._head_end.subject.navigability is True:
                 self.draw_head = self.draw_head_navigable
-            elif self._head_end.navigability == False:
+            elif self._head_end.subject.navigability is False:
                 self.draw_head = self.draw_head_none
             else:
                 self.draw_head = self.draw_head_undefined
@@ -176,9 +180,9 @@ class AssociationItem(NamedLine):
                 self.draw_tail = self.draw_tail_composite
             elif head_subject.aggregation == intern('shared'):
                 self.draw_tail = self.draw_tail_shared
-            elif self._tail_end.navigability:
+            elif self._tail_end.subject.navigability is True:
                 self.draw_tail = self.draw_tail_navigable
-            elif self._tail_end.navigability == False:
+            elif self._tail_end.subject.navigability is False:
                 self.draw_tail = self.draw_tail_none
             else:
                 self.draw_tail = self.draw_tail_undefined
@@ -204,13 +208,13 @@ class AssociationItem(NamedLine):
                                      handles[-2].pos)
         
 
-    def point(self, x, y):
+    def point(self, pos):
         """
         Returns the distance from the Association to the (mouse) cursor.
         """
-        return min(super(AssociationItem, self).point(x, y),
-                   self._head_end.point(x, y),
-                   self._tail_end.point(x, y))
+        return min(super(AssociationItem, self).point(pos),
+                   self._head_end.point(pos),
+                   self._tail_end.point(pos))
 
     def draw_head_none(self, context):
         """
@@ -406,7 +410,7 @@ class AssociationEnd(UML.Presentation):
         """
         if self.subject:
             try:
-                n, m = self.subject.render()
+                n, m = UML.format(self.subject)
             except ValueError:
                 # need more than 0 values to unpack: property was rendered as
                 # attribute while in a UNDO action for example.
@@ -416,127 +420,19 @@ class AssociationEnd(UML.Presentation):
                 self._mult = m
                 self.request_update()
 
-    def _get_navigability(self):
-        """
-        Check navigability of the AssociationEnd. If property is owned by
-        class via ownedAttribute, then it is navigable. If property is
-        owned by association by ownedEnd, then it is not navigable.
-        Otherwise the navigability is unknown.
 
-        Returned navigability values:
-            - None  - unknown
-            - False - not navigable
-            - True  - navigable
-        """
-        navigability = None # unknown navigability as default
-        subject = self.subject
-
-        if subject and subject.opposite:
-            #
-            # WARNING! see bug http://gaphor.devjavu.com/ticket/110
-            #
-            opposite = subject.opposite
-            if isinstance(opposite.type, UML.Interface):
-                type = subject.interface_
-            elif isinstance(opposite.type, UML.Class):
-                type = subject.class_
-            elif isinstance(opposite.type, UML.Actor):
-                type = subject.actor
-            elif isinstance(opposite.type, UML.UseCase):
-                type = subject.useCase
-            else:
-                assert 0, 'Should never be reached'
-
-            if type and subject in type.ownedAttribute:
-                navigability = True
-            elif subject.association and subject in subject.association.ownedEnd:
-                navigability = False
-
-        return navigability
-                
-
-    def _set_navigability(self, navigable):
-        """
-        Change the AssociationEnd's navigability.
-
-        A warning is issued if the subject or opposite property is missing.
-        """
-        subject = self.subject
-        if subject and subject.opposite:
-            opposite = subject.opposite
-
-            #
-            # Remove any navigability info, so unknown navigability state
-            # is the default.
-            #
-
-            # if navigable
-            #
-            # WARNING! see bug http://gaphor.devjavu.com/ticket/110
-            #
-            if isinstance(opposite.type, UML.Class):
-                if subject.class_:
-                    del subject.class_
-            elif isinstance(opposite.type, UML.Interface):
-                if subject.interface_:
-                    del subject.interface_
-            elif isinstance(opposite.type, UML.Actor):
-                if subject.actor:
-                    del subject.actor
-            elif isinstance(opposite.type, UML.UseCase):
-                if subject.useCase:
-                    del subject.useCase
-            else:
-                assert 0, 'Should never be reached'
-
-            # if not navigable
-            if subject.owningAssociation:
-                del subject.owningAssociation
-
-
-            #
-            # Set navigability.
-            #
-            #
-            # WARNING! see bug http://gaphor.devjavu.com/ticket/110
-            #
-            if navigable:
-                if isinstance(opposite.type, UML.Class):
-                    subject.class_ = opposite.type
-                elif isinstance(opposite.type, UML.Interface):
-                    subject.interface_ = opposite.type
-                elif isinstance(opposite.type, UML.Actor):
-                    subject.actor = opposite.type
-                elif isinstance(opposite.type, UML.UseCase):
-                    subject.useCase = opposite.type
-                else:
-                    assert 0, 'Should never be reached'
-            elif navigable == False:
-                subject.owningAssociation = subject.association
-
-            # else navigability is unknown
-
-        else:
-            log.warning('AssociationEnd.set_navigable: %s missing' % \
-                        (subject and 'subject' or 'opposite Property'))
-
-    navigability = property(_get_navigability, _set_navigability)
-
-
-    def point_name(self, x, y):
-        p = (x, y)
+    def point_name(self, pos):
         drp = distance_rectangle_point
-        return drp(self._name_bounds, p)
+        return drp(self._name_bounds, pos)
 
 
-    def point_mult(self, x, y):
-        p = (x, y)
+    def point_mult(self, pos):
         drp = distance_rectangle_point
-        return drp(self._mult_bounds, p)
+        return drp(self._mult_bounds, pos)
 
 
-    def point(self, x, y):
-        return min(self.point_name(x, y), self.point_mult(x, y))
+    def point(self, pos):
+        return min(self.point_name(pos), self.point_mult(pos))
 
 
     def get_name(self):
@@ -630,16 +526,15 @@ class AssociationEnd(UML.Presentation):
                                       height=mult_h)
 
 
-    def point(self, x, y):
+    def point(self, pos):
         """Given a point (x, y) return the distance to the canvas item.
         """
-        p = (x, y)
         drp = distance_rectangle_point
-        d1 = drp(self._name_bounds, p)
-        d2 = drp(self._mult_bounds, p)
+        d1 = drp(self._name_bounds, pos)
+        d2 = drp(self._mult_bounds, pos)
 #        try:
-#            d3 = geometry.distance_point_point(self._point1, p)
-#            d4, dummy = distance_line_point(self._point1, self._point2, p, 1.0, 0) #diacanvas.shape.CAP_ROUND)
+#            d3 = geometry.distance_point_point(self._point1, pos)
+#            d4, dummy = distance_line_point(self._point1, self._point2, pos, 1.0, 0) #diacanvas.shape.CAP_ROUND)
 #            if d3 < 15 and d4 < 5:
 #                d3 = 0.0
 #        except Exception, e:
