@@ -15,11 +15,10 @@ from gaphor.diagram.style import get_text_point_at_line, \
     ALIGN_CENTER, ALIGN_LEFT, ALIGN_RIGHT, ALIGN_TOP
 
 
-class LineItem(gaphas.Line, DiagramItem):
+class DiagramLine(gaphas.Line, DiagramItem):
     """
     Base class for diagram lines.
     """
-
     def __init__(self, id = None):
         gaphas.Line.__init__(self)
         DiagramItem.__init__(self, id)
@@ -43,62 +42,61 @@ class LineItem(gaphas.Line, DiagramItem):
         # first, update stereotype to know its text
         self.update_stereotype()
 
-        #super(LineItem, self).pre_update(context)
         gaphas.Line.pre_update(self, context)
         DiagramItem.pre_update(self, context)
 
 
     def post_update(self, context):
-        #super(LineItem, self).update(context)
         gaphas.Line.post_update(self, context)
         DiagramItem.post_update(self, context)
 
 
     def draw(self, context):
-        #super(LineItem, self).draw(context)
         gaphas.Line.draw(self, context)
         DiagramItem.draw(self, context)
 
 
-    def point(self, x, y):
-        d1 = gaphas.Line.point(self, x, y)
-        d2 = DiagramItem.point(self, x, y)
+    def point(self, pos):
+        d1 = gaphas.Line.point(self, pos)
+        d2 = DiagramItem.point(self, pos)
         return min(d1, d2)
 
 
-class DiagramLine(LineItem):
-    """
-    Gaphor lines. This line is serializable and has a popup
-    menu.
-
-    TODO: put serializability and popup in separate adapters.
-    """
-
-    def save (self, save_func):
-        LineItem.save(self, save_func)
+    def save(self, save_func):
+        DiagramItem.save(self, save_func)
         save_func('matrix', tuple(self.matrix))
         for prop in ('orthogonal', 'horizontal'):
             save_func(prop, getattr(self, prop))
-        points = [ ]
+        points = []
         for h in self.handles():
             points.append(tuple(map(float, h.pos)))
         save_func('points', points)
-        c = self.head.connected_to
-        if c:
-            save_func('head-connection', c, reference=True)
-        c = self.tail.connected_to
-        if c:
-            save_func('tail-connection', c, reference=True)
 
-    def load (self, name, value):
+        canvas = self.canvas
+        c = canvas.get_connection(self.head)
+        if c:
+            save_func('head-connection', c.connected, reference=True)
+        c = canvas.get_connection(self.tail)
+        if c:
+            save_func('tail-connection', c.connected, reference=True)
+
+
+    def load(self, name, value):
         if name == 'matrix':
             self.matrix = eval(value)
         elif name == 'points':
             points = eval(value)
             for x in xrange(len(points) - 2):
-                self.split_segment(0)
+                h = self._create_handle((0, 0))
+                self._handles.insert(1, h)
             for i, p in enumerate(points):
                 self.handles()[i].pos = p
+
+            # Update connection ports of the line. Only handles are saved
+            # in Gaphor file therefore ports need to be recreated after
+            # handles information is loaded.
+            self._update_ports()
+
         elif name == 'orthogonal':
             self._load_orthogonal = eval(value)
         elif name in ('head_connection', 'head-connection'):
@@ -106,11 +104,46 @@ class DiagramLine(LineItem):
         elif name in ('tail_connection', 'tail-connection'):
             self._load_tail_connection = value
         else:
-            LineItem.load(self, name, value)
+            DiagramItem.load(self, name, value)
+
+
+    def _get_sink(self, handle, item):
+        """
+        Instant port finder.
+
+        This is not the nicest place for such method.
+        
+        TODO: figure out if part of this functionality can be provided by
+        the storage code.
+        """
+        from gaphas.aspect import ConnectionSink
+
+        hpos = self.canvas.get_matrix_i2i(self, item).transform_point(*handle.pos)
+        port = None
+        dist = 10e6
+        for p in item.ports():
+            pos, d = p.glue(hpos)   
+            if not port or d < dist:
+                port = p
+                dist = d
+                
+        return ConnectionSink(item, port)
+
+
+    def _postload_connect(self, handle, item):
+        """
+        Postload connect method.
+        """
+        from gaphas.aspect import Connector
+
+        connector = Connector(self, handle)
+
+        sink = self._get_sink(handle, item)
+
+        connector.connect(sink)
+
 
     def postload(self):
-        # Ohoh, need the IConnect adapters here
-        from zope import component
         if hasattr(self, '_load_orthogonal'):
             self.orthogonal = self._load_orthogonal
             del self._load_orthogonal
@@ -118,23 +151,18 @@ class DiagramLine(LineItem):
         # First update matrix and solve constraints (NE and SW handle are
         # lazy and are resolved by the constraint solver rather than set
         # directly.
-        #self.canvas.update_matrix(self)
-        #self.canvas.solver.solve()
+        self.canvas.update_matrix(self)
+        self.canvas.solver.solve()
 
         if hasattr(self, '_load_head_connection'):
-            adapter = component.queryMultiAdapter((self._load_head_connection, self), IConnect)
-            assert adapter, 'No IConnect adapter to connect %s to %s' % (self._load_head_connection, self)
-
-            adapter.connect(self.head)
+            self._postload_connect(self.head, self._load_head_connection)
             del self._load_head_connection
 
         if hasattr(self, '_load_tail_connection'):
-            adapter = component.queryMultiAdapter((self._load_tail_connection, self), IConnect)
-            assert adapter, 'No IConnect adapter to connect %s to %s' % (self._load_tail_connection, self)
-
-            adapter.connect(self.tail)
+            self._postload_connect(self.tail, self._load_tail_connection)
             del self._load_tail_connection
-        LineItem.postload(self)
+
+        DiagramItem.postload(self)
 
 
     def _get_middle_segment(self):
@@ -153,8 +181,8 @@ class DiagramLine(LineItem):
         the middle segment is also returned.
         """
         h0, h1 = self._get_middle_segment()
-        pos = (h0.x + h1.x) / 2, (h0.y + h1.y) / 2
-        angle = atan2(h1.y - h0.y, h1.x - h0.x)
+        pos = (h0.pos.x + h1.pos.x) / 2, (h0.pos.y + h1.pos.y) / 2
+        angle = atan2(h1.pos.y - h0.pos.y, h1.pos.x - h0.pos.x)
         if inverted:
             angle += pi
         return pos, angle
@@ -203,47 +231,16 @@ class NamedLine(DiagramLine):
                     'text-align-str': self.style.name_align_str,
                     'text-align-group': 'stereotype',
                 }, editable=True)
-        self.add_watch(UML.NamedElement.name, self.on_named_element_name)
+        self.watch('subject<NamedElement>.name', self.on_named_element_name)
 
 
+    def postload(self):
+        super(NamedLine, self).postload()
+        self.on_named_element_name(None)
+
     def on_named_element_name(self, event):
-        if self.subject and (event is None or self.subject is event.element):
-            self._name.text = self.subject.name
-            self.request_update()
+        self._name.text = self.subject and self.subject.name or ''
+        self.request_update()
 
 
-class FreeLine(LineItem):
-    """
-    TODO: get rid of this one.
-
-    A line with disabled last handle. This allows to create diagram items,
-    which have one or more additional lines.
-
-    Last handle is usually attached to a point of diagram item. This point
-    is called main point.
-    """
-    def __init__(self, id = None, mpt = None):
-        LineItem.__init__(self, id)
-
-        # expose first handle
-        self._handle = self.handles[0]
-
-        # disable last handle, it should be in main point
-        h = self.handles[-1]
-        h.props.movable = False
-        h.props.connectable = False
-        h.props.visible = False
-
-        self._main_point = mpt
-
-        if self._main_point is None:
-            self._main_point = 0, 0
-
-
-    def on_update(self, affine):
-        self.handles[-1].set_pos_i(*self._main_point)
-        self._handle.set_pos_i(*self._handle.get_pos_i()) # really strange, but we have to do this
-
-        LineItem.on_update(self, affine)
-
-# vim:sw=4:et
+# vim:sw=4:et:ai
