@@ -6,7 +6,7 @@ See also gaphor.diagram.states package description.
 
 from gaphor import UML
 from gaphor.diagram.style import ALIGN_LEFT, ALIGN_TOP
-from gaphas.util import path_ellipse
+from gaphas.util import path_ellipse, text_center
 
 from gaphor.diagram.states import VertexItem
 
@@ -14,9 +14,6 @@ from gaphor.diagram.states import VertexItem
 class InitialPseudostateItem(VertexItem):
     """
     Initial pseudostate diagram item.
-
-    Parameters
-        - _connected - guard used to not connect more than one transition
     """
     __uml__   = UML.Pseudostate
     __style__ = {
@@ -32,8 +29,6 @@ class InitialPseudostateItem(VertexItem):
         for h in self.handles():
             h.movable = False
 
-        self._connected = False
-
 
     def draw(self, context):
         """
@@ -47,4 +42,37 @@ class InitialPseudostateItem(VertexItem):
         cr.set_line_width(0.01)
         cr.fill()
 
+
+class HistoryPseudostateItem(VertexItem):
+    """
+    History pseudostate diagram item.
+    """
+    __uml__   = UML.Pseudostate
+    __style__ = {
+        'min-size':   (30, 30),
+        'name-align': (ALIGN_LEFT, ALIGN_TOP),
+        'name-padding': (2, 2, 2, 2),
+        'name-outside': True,
+    }
+
+    RADIUS = 15
+    def __init__(self, id=None):
+        super(HistoryPseudostateItem, self).__init__(id)
+        for h in self.handles():
+            h.movable = False
+
+
+    def draw(self, context):
+        """
+        Draw intial pseudostate symbol.
+        """
+        super(HistoryPseudostateItem, self).draw(context)
+        cr = context.cairo
+        r = self.RADIUS
+        d = r * 2
+        path_ellipse(cr, r, r, d, d)
+        #cr.set_line_width(1)
+        cr.stroke()
+        text_center(cr, r, r, "H")
+
 # vim:sw=4:et
