@@ -4,7 +4,8 @@ Copy / Paste functionality
 
 from zope import interface, component
 import gaphas
-from gaphor import UML
+from gaphor.UML import Element
+from gaphor.UML.collection import collection
 from gaphor.interfaces import IService, IActionProvider
 from gaphor.ui.interfaces import IDiagramSelectionChange
 from gaphor.core import _, inject, action, build_action_group, transactional
@@ -69,28 +70,30 @@ class CopyService(object):
             self.copy_buffer = set(items)
             self.action_group.get_action('edit-paste').props.sensitive = True
 
-    def _load_element(self, name, value):
+
+    def copy_func(self, name, value, reference=False):
         """
         Copy an element, preferbly from the list of new items,
         otherwise from the element factory.
         If it does not exist there, do not copy it!
         """
-        item = self._new_items.get(value.id)
-        if item:
-            self._item.load(name, item)
-        else:
-            item = self.element_factory.lookup(value.id)
+        def load_element():
+            item = self._new_items.get(value.id)
             if item:
                 self._item.load(name, item)
+            else:
+                item = self.element_factory.lookup(value.id)
+                if item:
+                    self._item.load(name, item)
 
-    def copy_func(self, name, value, reference=False):
-        if reference or isinstance(value, UML.Element):
-            self._load_element(name, value)
-        elif isinstance(value, UML.collection):
-            for item in value:
-                self._load_element(name, item)
+        if reference or isinstance(value, Element):
+            load_element()
+        elif isinstance(value, collection):
+            values = value
+            for value in values:
+                load_element()
         elif isinstance(value, gaphas.Item):
-            self._load_element(name, value)
+            load_element()
         else:
             # Plain attribute
             self._item.load(name, str(value))
@@ -136,7 +139,7 @@ class CopyService(object):
 
 
     @action(name='edit-copy', stock_id='gtk-copy')
-    def _copy(self):
+    def copy_action(self):
         view = self.gui_manager.main_window.get_current_diagram_view()
         if view.is_focus():
             items = view.selected_items
