@@ -10,7 +10,7 @@ from gaphor.core import _, inject, action, build_action_group
 from gaphor import UML
 from gaphor.misc.gidlethread import GIdleThread, Queue, QueueEmpty
 from gaphor.misc.xmlwriter import XMLWriter
-from gaphor.misc.errorhandler import error_handler, ErrorHandlerAspect, weave_method
+from gaphor.misc.errorhandler import error_handler
 DEFAULT_EXT='.gaphor'
 
 class FileManagerStateChanged(object):
@@ -61,11 +61,13 @@ class FileManager(object):
           </menu>
         </menubar>
         <toolbar action="mainwindow-toolbar">
-          <toolitem action="file-open" />
-          <separator />
-          <toolitem action="file-save" />
-          <toolitem action="file-save-as" />
-          <separator />
+          <placeholder name="left">
+            <toolitem action="file-open" />
+            <separator />
+            <toolitem action="file-save" />
+            <toolitem action="file-save-as" />
+            <separator />
+          </placeholder>
         </toolbar>
       </ui>
     """
@@ -181,6 +183,23 @@ class FileManager(object):
 
     def _save(self, filename):
         if filename and len(filename) > 0:
+            from gaphor.storage import verify
+            orphans = verify.orphan_references(self.element_factory)
+            if orphans:
+                main_window = self.gui_manager.main_window
+                dialog = gtk.MessageDialog(main_window.window,
+                    gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                    gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO,
+                    _("The model contains some references to items that are "
+                    "not maintained. Do you want to clean this before saving the model?"))
+                #dialog.format_secondary_text("The 
+                answer = dialog.run()
+                dialog.destroy()
+                if answer != gtk.RESPONSE_YES:
+                    for o in orphans:
+                        o.unlink()
+                    assert not verify.orphan_references(self.element_factory)
+
             from gaphor.storage import storage
             if not filename.endswith(DEFAULT_EXT):
                 filename = filename + DEFAULT_EXT
@@ -211,7 +230,11 @@ class FileManager(object):
     def _open_dialog(self, title):
         filesel = gtk.FileChooserDialog(title=title,
                                         action=gtk.FILE_CHOOSER_ACTION_OPEN,
-                                        buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
+                                        buttons=(gtk.STOCK_CANCEL,
+                                                 gtk.RESPONSE_CANCEL,
+                                                 gtk.STOCK_OPEN,
+                                                 gtk.RESPONSE_OK))
+        filesel.set_transient_for(self.gui_manager.main_window.window)
 
         filter = gtk.FileFilter()
         filter.set_name("Gaphor models")
@@ -268,7 +291,12 @@ class FileManager(object):
         filename = self.filename
         filesel = gtk.FileChooserDialog(title=_('Save Gaphor model as'),
                                         action=gtk.FILE_CHOOSER_ACTION_SAVE,
-                                        buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_SAVE,gtk.RESPONSE_OK))
+                                        buttons=(gtk.STOCK_CANCEL,
+                                                 gtk.RESPONSE_CANCEL,
+                                                 gtk.STOCK_SAVE,
+                                                 gtk.RESPONSE_OK))
+        filesel.set_transient_for(self.gui_manager.main_window.window)
+
         if filename:
             filesel.set_current_name(filename)
         response = filesel.run()
