@@ -6,9 +6,6 @@ load(filename)
     load a model from a file
 save(filename)
     store the current model in a file
-verify(filename)
-    check the validity of the file (this does not tell us
-    we have a valid model, just a valid file).
 """
 
 from cStringIO import StringIO, InputType
@@ -21,6 +18,7 @@ import gc
 import gaphas
 
 from gaphor import UML
+from gaphor.UML.collection import collection
 from gaphor.UML.elementfactory import ElementChangedEventBlocker
 from gaphor import diagram
 from gaphor.storage import parser
@@ -39,6 +37,7 @@ from gaphor.adapters import connectors
 __all__ = [ 'load', 'save' ]
 
 FILE_FORMAT_VERSION = '3.0'
+NAMESPACE_MODEL = 'http://gaphor.sourceforge.net/model'
 
 def save(writer=None, factory=None, status_queue=None):
     for status in save_generator(writer, factory):
@@ -52,13 +51,16 @@ def save_generator(writer, factory):
     with CDATA support).
     """
 
+    # Maintain a set of id's, one for elements, one for references.
+    # Write only to file if references is a subset of elements
+
     def save_reference(name, value):
         """
         Save a value as a reference to another element in the model.
         This applies to both UML as well as canvas items.
         """
         # Save a reference to the object:
-        if value.id: #, 'Referenced element %s has no id' % value
+        if value.id:
             writer.startElement(name, {})
             writer.startElement('ref', { 'refid': value.id })
             writer.endElement('ref')
@@ -73,7 +75,7 @@ def save_generator(writer, factory):
             writer.startElement('reflist', {})
             for v in value:
                 #save_reference(name, v)
-                if v.id: #, 'Referenced element %s has no id' % v
+                if v.id:
                     writer.startElement('ref', { 'refid': v.id })
                     writer.endElement('ref')
             writer.endElement('reflist')
@@ -109,7 +111,7 @@ def save_generator(writer, factory):
         #log.debug('saving element: %s|%s %s' % (name, value, type(value)))
         if isinstance (value, (UML.Element, gaphas.Item)):
             save_reference(name, value)
-        elif isinstance(value, UML.collection):
+        elif isinstance(value, collection):
             save_collection(name, value)
         elif isinstance(value, gaphas.Canvas):
             writer.startElement('canvas', {})
@@ -124,7 +126,7 @@ def save_generator(writer, factory):
         The extra attribute reference can be used to force UML 
         """
         #log.debug('saving canvasitem: %s|%s %s' % (name, value, type(value)))
-        if isinstance(value, UML.collection) or \
+        if isinstance(value, collection) or \
                 (isinstance(value, (list, tuple)) and reference == True):
             save_collection(name, value)
         elif reference:
@@ -137,10 +139,6 @@ def save_generator(writer, factory):
             # save subitems
             for child in value.canvas.get_children(value):
                 save_canvasitem(None, child)
-#                writer.startElement('item', { 'id': child.id,
-#                                              'type': kid.__class__.__name__ })
-#                child.save(save_canvasitem)
-#                writer.endElement('item')
 
             writer.endElement('item')
 
@@ -150,8 +148,10 @@ def save_generator(writer, factory):
             save_value(name, value)
 
     writer.startDocument()
-    writer.startElement('gaphor', { 'version': FILE_FORMAT_VERSION,
-                                    'gaphor-version': Application.distribution.version })
+    writer.startPrefixMapping('', NAMESPACE_MODEL)
+    writer.startElementNS((NAMESPACE_MODEL, 'gaphor'), None,
+            { (NAMESPACE_MODEL, 'version'): FILE_FORMAT_VERSION,
+              (NAMESPACE_MODEL, 'gaphor-version'): Application.distribution.version })
 
     size = factory.size()
     n = 0
@@ -166,7 +166,9 @@ def save_generator(writer, factory):
         if n % 25 == 0:
             yield (n * 100) / size
 
-    writer.endElement('gaphor')
+    #writer.endElement('gaphor')
+    writer.endElementNS((NAMESPACE_MODEL, 'gaphor'), None)
+    writer.endPrefixMapping('')
     writer.endDocument()
 
 
@@ -196,11 +198,25 @@ def load_elements_generator(elements, factory, gaphor_version=None):
     version_0_6_2(elements, factory, gaphor_version)
     version_0_7_2(elements, factory, gaphor_version)
     version_0_9_0(elements, factory, gaphor_version)
+    version_0_14_0(elements, factory, gaphor_version)
+    version_0_15_0_pre(elements, factory, gaphor_version)
 
     #log.debug("Still have %d elements" % len(elements))
 
     # First create elements and canvas items in the factory
     # The elements are stored as attribute 'element' on the parser objects:
+
+    def create_canvasitems(canvas, canvasitems, parent=None):
+        """
+        Canvas is a read gaphas.Canvas, items is a list of parser.canvasitem's
+        """
+        for item in canvasitems:
+            cls = getattr(items, item.type)
+            item.element = diagram.create_as(cls, item.id)
+            canvas.add(item.element, parent=parent)
+            assert canvas.get_parent(item.element) is parent
+            create_canvasitems(canvas, item.canvasitems, parent=item.element)
+
     for id, elem in elements.items():
         st = update_status_queue()
         if st: yield st
@@ -210,17 +226,10 @@ def load_elements_generator(elements, factory, gaphor_version=None):
             elem.element = factory.create_as(cls, id)
             if elem.canvas:
                 elem.element.canvas.block_updates = True
-        elif isinstance(elem, parser.canvasitem):
-            cls = getattr(items, elem.type)
-            #log.debug('Creating canvas item for %s (%s)' % (elem, elem.id))
-            elem.element = diagram.create_as(cls, id)
-        else:
+                create_canvasitems(elem.element.canvas, elem.canvas.canvasitems)
+        elif not isinstance(elem, parser.canvasitem):
             raise ValueError, 'Item with id "%s" and type %s can not be instantiated' % (id, type(elem))
 
-    #log.info('0% ... 33%')
-
-    #log.debug("Still have %d elements" % len(elements))
-
     # load attributes and create references:
     for id, elem in elements.items():
         st = update_status_queue()
@@ -228,18 +237,6 @@ def load_elements_generator(elements, factory, gaphor_version=None):
         # Ensure that all elements have their element instance ready...
         assert hasattr(elem, 'element')
 
-        # establish parent/child relations on canvas items:
-        if isinstance(elem, parser.element) and elem.canvas:
-            for item in elem.canvas.canvasitems:
-                assert item in elements.values(), 'Item %s (%s) is a canvas item, but it is not in the parsed objects table' % (item, item.id)
-                elem.element.canvas.add(item.element)
-
-        # Also create nested canvas items:
-        if isinstance(elem, parser.canvasitem):
-            for item in elem.canvasitems:
-                assert item in elements.values(), 'Item %s (%s) is a canvas item, but it is not in the parsed objects table' % (item, item.id)
-                elem.element.canvas.add(item.element, parent=elem.element)
-
         # load attributes and references:
         for name, value in elem.values.items():
             try:
@@ -276,6 +273,7 @@ def load_elements_generator(elements, factory, gaphor_version=None):
     # Fix version inconsistencies
     version_0_5_2(elements, factory, gaphor_version)
     version_0_7_1(elements, factory, gaphor_version)
+    version_0_15_0_post(elements, factory, gaphor_version)
 
     # Before version 0.7.2 there was only decision node (no merge nodes).
     # This node could have many incoming and outgoing flows (edges).
@@ -363,6 +361,290 @@ def load_generator(filename, factory):
         log.info('file %s could not be loaded' % filename, e)
         raise
 
+def version_lower_than(gaphor_version, version):
+    """
+    if version_lower_than('0.3.0', (0, 15, 0)):
+       ...
+    """
+    parts = gaphor_version.split('.')
+    try:
+        return tuple(map(int, parts)) < version
+    except ValueError:
+        # We're having a -pre, -beta, -alpha or whatever version
+        parts = parts[:-1]
+        return tuple(map(int, parts)) <= version
+    
+
+def version_0_15_0_pre(elements, factory, gaphor_version):
+    """
+    Fix association navigability UML metamodel to comply with UML 2.2
+    using Association.navigableOwnedEnd among others (see model factory
+    for details).
+
+    Convert tagged values into comment items as tagged values are no longer
+    supported by UML specification (stereotypes attributes shall be used
+    instead). Comment item contains information about used tagged values.
+    It means, that full conversion of tagged values into stereotype
+    attributes is not supported at the moment.
+
+    This function is called before the actual elements are constructed.
+    """
+    ATTRS = set(['class_', 'interface_', 'actor', 'useCase', 'owningAssociation'])
+    if version_lower_than(gaphor_version, (0, 14, 99)):
+        # update associations
+        values = (v for v in elements.values()
+                if type(v) is parser.element
+                    and v.type == 'Property'
+                    and 'association' in v.references)
+        for et in values:
+            # get association
+            assoc = elements[et.references['association']]
+
+            attrs = set(set(ATTRS) & set(et.references))
+            if attrs:
+                assert len(attrs) == 1
+
+                attr = attrs.pop()
+
+                if attr == 'owningAssociation':
+                    assoc.references['ownedEnd'].remove(et.id)
+                    if not assoc.references['ownedEnd']:
+                        del assoc.references['ownedEnd']
+                elif attr in ('actor', 'useCase'):
+                    if 'navigableOwnedEnd' not in assoc.references:
+                        assoc.references['navigableOwnedEnd'] = []
+                    assoc.references['navigableOwnedEnd'].append(et.id)
+
+                    el = elements[et.references[attr]]
+                    el.references['ownedAttribute'].remove(et.id)
+                    if not el.references['ownedAttribute']:
+                        del el.references['ownedAttribute']
+
+                del et.references[attr]
+            else:
+                if 'ownedEnd' not in assoc.references:
+                    assoc.references['ownedEnd'] = []
+                assoc.references['ownedEnd'].append(et.id)
+
+        # - get rid of tagged values
+        for e in elements.values():
+            if 'taggedValue' in e.references:
+                taggedvalue = [elements[i].values['value'] for i in e.references['taggedValue'] if elements[i].values.get('value')]
+                #convert_tagged_value(e, elements, factory)
+                if taggedvalue:
+                    e.taggedvalue = taggedvalue
+                
+                # Remove obsolete elements
+                for t in e.references['taggedValue']:
+                    del elements[t]
+                del e.references['taggedValue']
+
+        # - rename EventOccurrence to MessageOccurrenceSpecification
+        values = (v for v in elements.values()
+                if type(v) is parser.element
+                    and v.type == 'EventOccurrence')
+        for et in values:
+            et.type = 'MessageOccurrenceSpecification'
+
+
+def version_0_15_0_post(elements, factory, gaphor_version):
+    """
+    Part two: create stereotypes and what more for the elements that have a
+    taggedvalue property.
+    """
+    def update_elements(element):
+        e = elements[element.id] = parser.element(element.id, element.__class__.__name__)
+        e.element = element
+
+    if version_lower_than(gaphor_version, (0, 14, 99)):
+        stereotypes = {}
+        profile = None
+        for e in elements.values():
+            if hasattr(e, 'taggedvalue'):
+                if not profile:
+                    profile = factory.create(UML.Profile)
+                    profile.name = 'version 0.15 conversion'
+                    update_elements(profile)
+                st = stereotypes.get(e.type)
+                if not st:
+                    st = stereotypes[e.type] = factory.create(UML.Stereotype)
+                    st.name = 'Tagged'
+                    st.package = profile
+                    update_elements(st)
+                    cl = factory.create(UML.Class)
+                    cl.name = str(e.type)
+                    cl.package = profile
+                    update_elements(cl)
+                    ext = UML.model.extend_with_stereotype(factory, cl, st)
+                    update_elements(ext)
+                    for me in ext.memberEnd: update_elements(me)
+                # Create instance specification for the stereotype:
+                instspec = UML.model.apply_stereotype(factory, e.element, st)
+                update_elements(instspec)
+
+                def create_slot(key, val):
+                    for attr in st.ownedAttribute:
+                        if attr.name == key:
+                            break
+                    else:
+                        attr = st.ownedAttribute = factory.create(UML.Property)
+                        attr.name = str(key)
+                        update_elements(attr)
+                    slot = UML.model.add_slot(factory, instspec, attr)
+                    slot.value.value = str(val)
+                    update_elements(slot)
+
+                tviter = iter(e.taggedvalue or [])
+                for tv in tviter:
+                    try:
+                        try:
+                            key, val = tv.split('=', 1)
+                            key = key.strip()
+                        except ValueError:
+                            log.info('Tagged value "%s" has no key=value format, trying key_value ' % tv)
+                            try:
+                                key, val = tv.split(' ', 1)
+                                key = key.strip()
+                            except ValueError:
+                                # Fallback, deal with it as if it were a boolean
+                                key = tv.strip()
+                                val = 'true'
+
+                            # This syntax is used with the UML meta model:
+                            if key in ('subsets', 'redefines'):
+                                rest = ', '.join(tviter)
+                                val = ', '.join([val, rest]) if rest else val
+                                val = val.replace('\n', ' ')
+                                log.info('Special case: UML metamodel "%s %s"' % (key, val))
+                        create_slot(key, val)
+                    except Exception, e:
+                        log.warning('Unable to process tagged value "%s" as key=value pair' % tv, e)
+
+        def find(messages, attr):
+            occurrences = set(getattr(m, attr) for m in messages
+                    if hasattr(m, attr) and getattr(m, attr))
+            assert len(occurrences) <= 1
+            if occurrences:
+                return occurrences.pop()
+            else:
+                return None
+
+        def update_msg(msg, sl, rl):
+            if sl:
+                s = factory.create(UML.MessageOccurrenceSpecification)
+                s.covered = sl
+                m.sendEvent = s
+            if rl:
+                r = factory.create(UML.MessageOccurrenceSpecification)
+                r.covered = rl
+                m.receiveEvent = r
+
+        for e in elements.values():
+            if e.type == 'MessageItem':
+                msg = e.element
+                send = msg.subject.sendEvent
+                receive = msg.subject.receiveEvent
+
+                if not send:
+                    send = find(msg._messages.keys(), 'sendEvent')
+                if not receive:
+                    receive = find(msg._messages.keys(), 'receiveEvent')
+                if not send:
+                    send = find(msg._inverted_messages.keys(), 'reveiveEvent')
+                if not receive:
+                    receive = find(msg._inverted_messages.keys(), 'sendEvent')
+
+                sl = send.covered if send else None
+                rl = receive.covered if receive else None
+
+                for m in msg._messages:
+                    update_msg(m, sl, rl)
+                for m in msg._inverted_messages:
+                    update_msg(m, rl, sl)
+                msg.subject.sendEvent = send
+                msg.subject.receiveEvent = receive
+
+
+def convert_tagged_value(element, elements, factory):
+    """
+    Convert ``element.taggedValue`` to something supported by the
+    UML 2.2 model (since Gaphor version 0.15).
+
+    For each item having a Tagged value a Stereotype is linked. This is done
+    like this:
+
+      item -> InstanceSpecification -> Stereotype
+
+    Each tagged value will be replaced by a Slot:
+
+      item -> InstanceSpecification -> Slot -> Attribute -> Stereotype
+    """
+    import uuid
+    diagrams = [e for e in elements.values() if e.type == 'Diagram']
+
+    presentation = element.get('presentation') or []
+    tv = [elements[i] for i in element.references['taggedValue']]
+    for et in presentation:
+        et = elements[et]
+        m = eval(et.values['matrix'])
+        w = eval(et.values['width'])
+
+        tagged = 'upgrade to stereotype attributes' \
+            ' following tagged values:\n%s' % '\n'.join(t.values['value'] for t in tv)
+
+        item = parser.canvasitem(str(uuid.uuid1()), 'CommentItem')
+        comment = parser.element(str(uuid.uuid1()), 'Comment')
+
+        item.references['subject'] = comment.id
+        item.values['matrix'] = str((1.0, 0.0, 0.0, 1.0, m[4] + w + 10.0, m[5]))
+
+        comment.references['presentation'] = [item.id]
+        comment.values['body'] = tagged
+
+        elements[item.id] = item
+        elements[comment.id] = comment
+
+        # Where to place the comment? How to find the Diagram?
+        for d in diagrams:
+            for ci in d.canvas.canvasitems:
+                if ci.id == et.id:
+                    d.canvas.canvasitems.append(item)
+                    break
+
+def version_0_14_0(elements, factory, gaphor_version):
+    """
+    Fix applied stereotypes UML metamodel. Before Gaphor 0.14.0 applied
+    stereotypes was a collection of stereotypes classes, but now the list
+    needs to be replaced with collection of stereotypes instances.
+
+    This function is called before the actual elements are constructed.
+    """
+    import uuid
+    if version_lower_than(gaphor_version, (0, 14, 0)):
+        values = (v for v in elements.values() if type(v) is parser.element)
+        for et in values:
+            try:
+                if 'appliedStereotype' in et.references:
+                    data = tuple(et.references['appliedStereotype'])
+                    applied = []
+                    # collect stereotypes instances in `applied` list
+                    for refid in data:
+                        st = elements[refid]
+                        obj = parser.element(str(uuid.uuid1()),
+                                             'InstanceSpecification')
+                        obj.references['classifier'] = [st.id]
+                        elements[obj.id] = obj
+                        applied.append(obj.id)
+
+                        assert obj.id in applied and obj.id in elements
+
+                    # replace stereotypes with their instances
+                    assert len(applied) == len(data)
+                    et.references['appliedStereotype'] = applied
+
+            except Exception, e:
+                log.error('Error while updating stereotypes', e)
+
 
 def version_0_9_0(elements, factory, gaphor_version):
     """
@@ -372,7 +654,7 @@ def version_0_9_0(elements, factory, gaphor_version):
 
     This function is called before the actual elements are constructed.
     """
-    if tuple(map(int, gaphor_version.split('.'))) < (0, 9, 0):
+    if version_lower_than(gaphor_version, (0, 9, 0)):
         for elem in elements.values():
             try:
                 if type(elem) is parser.canvasitem:
@@ -385,7 +667,7 @@ def version_0_9_0(elements, factory, gaphor_version):
                         del elem.values['color']
 
             except Exception, e:
-                log.error('Error while updating taggedValues', e)
+                log.error('Error while updating from DiaCanvas2', e)
 
 def version_0_7_2(elements, factory, gaphor_version):
     """
@@ -394,9 +676,9 @@ def version_0_7_2(elements, factory, gaphor_version):
     the multiplicity of taggedValue has changed from 0..1 to *, so all elements
     should be converted to a list.
     """
-    from gaphor.misc.uniqueid import generate_id
+    import uuid
 
-    if tuple(map(int, gaphor_version.split('.'))) < (0, 7, 2):
+    if version_lower_than(gaphor_version, (0, 7, 2)):
         for elem in elements.values():
             try:
                 if type(elem) is parser.element \
@@ -407,7 +689,7 @@ def version_0_7_2(elements, factory, gaphor_version):
                     if tv.value:
                         for t in map(str.strip, str(tv.value).split(',')):
                             #log.debug("Tagged value: %s" % t)
-                            newtv = parser.element(generate_id(),
+                            newtv = parser.element(str(uuid.uuid1()),
                                                    'LiteralSpecification')
                             newtv.values['value'] = t
                             elements[newtv.id] = newtv
@@ -438,7 +720,7 @@ def version_0_7_1(elements, factory, gaphor_version):
         if not (type and end1 in type.ownedAttribute):
             del end1.owningAssociation
 
-    if tuple(map(int, gaphor_version.split('.'))) < (0, 7, 1):
+    if version_lower_than(gaphor_version, (0, 7, 1)):
         log.info('Fix navigability of Associations (file version: %s)' % gaphor_version)
         for elem in elements.values():
             try:
@@ -458,7 +740,7 @@ def version_0_6_2(elements, factory, gaphor_version):
     Before 0.6.2 an Interface could be represented by a ClassItem and
     a InterfaceItem. Now only InterfaceItems are used.
     """
-    if tuple(map(int, gaphor_version.split('.'))) < (0, 6, 2):
+    if version_lower_than(gaphor_version, (0, 6, 2)):
         for elem in elements.values():
             try:
                 if type(elem) is parser.element and elem.type == 'Interface':
@@ -478,7 +760,7 @@ def version_0_5_2(elements, factory, gaphor_version):
     Before version 0.5.2, the wrong memberEnd of the association was
     holding the aggregation information.
     """
-    if tuple(map(int, gaphor_version.split('.'))) < (0, 5, 2):
+    if version_lower_than(gaphor_version, (0, 5, 2)):
         log.info('Fix composition on Associations (file version: %s)' % gaphor_version)
         for elem in elements.values():
             try:
@@ -492,4 +774,4 @@ def version_0_5_2(elements, factory, gaphor_version):
                 log.error('Error while updating Association', e)
 
 
-# vim: sw=4:et
+# vim: sw=4:et:ai
