@@ -6,11 +6,13 @@ from zope import interface, component
 from gaphor.interfaces import ITransaction
 from gaphor.event import TransactionBegin, TransactionCommit, TransactionRollback
 
+
 def transactional(func):
     def _transactional(*args, **kwargs):
+        r = None
         tx = Transaction()
         try:
-	    func(*args, **kwargs)
+            r = func(*args, **kwargs)
         except Exception, e:
             log.error('Transaction terminated due to an exception, performing a rollback', e)
             try:
@@ -20,8 +22,10 @@ def transactional(func):
             raise
         else:
             tx.commit()
+        return r
     return _transactional
 
+
 class TransactionError(Exception):
     """
     Errors related to the transaction module.
@@ -29,6 +33,29 @@ class TransactionError(Exception):
 
 
 class Transaction(object):
+    """
+    The transaction. On start and end of a transaction an event is emited.
+
+    Transactions can be nested. If the outermost transaction is committed or
+    rolled back, an event is emitted.
+
+    Events can be handled programmatically:
+
+    >>> tx = Transaction()
+    >>> tx.commit()
+
+    It can be assigned as decorator:
+
+    >>> @transactional
+    ... def foo():
+    ...     pass
+
+    Or with the ``with`` statement:
+
+    >>> with Transaction:
+    ...     pass
+    """
+
     interface.implements(ITransaction)
 
     _stack= []
@@ -41,8 +68,6 @@ class Transaction(object):
 
     def commit(self):
         self._close()
-#        if self._need_rollback:
-#            log.warning('Tried to commit a transaction already marked for rollback.')
         if not self._stack:
             if self._need_rollback:
                 component.handle(TransactionRollback())
@@ -66,5 +91,13 @@ class Transaction(object):
             self._stack.append(last)
             raise TransactionError, 'Transaction on stack is not the transaction being closed.'
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type=None, exc_val=None, exc_tb=None):
+        if exc_type:
+            self.rollback()
+        else:
+            self.commit()
 
 # vim: sw=4:et:ai
