@@ -1,10 +1,10 @@
 #!/usr/bin/env python
-# vim: sw=4:et
 
 import gtk
 from cairo import Matrix
 
 from zope import component
+from gaphas.view import GtkView
 from gaphor import UML
 from gaphor.core import _, inject, transactional, action, build_action_group
 from gaphor.application import Application
@@ -12,7 +12,6 @@ from gaphor.UML.interfaces import IAttributeChangeEvent, IElementDeleteEvent
 from gaphor.diagram import get_diagram_item
 from gaphor.diagram.items import DiagramItem
 from gaphor.transaction import Transaction
-from gaphor.ui.diagramview import DiagramView
 from gaphor.ui.diagramtoolbox import DiagramToolbox
 from event import DiagramSelectionChange
 
@@ -25,10 +24,13 @@ class DiagramTab(object):
       <ui>
         <menubar action="mainwindow">
           <menu action="edit">
-            <menuitem action="diagram-delete" />
-            <separator />
-            <menuitem action="diagram-select-all" />
-            <menuitem action="diagram-unselect-all" />
+            <placeholder name="ternary">
+              <menuitem action="diagram-delete" />
+              <separator />
+              <menuitem action="diagram-select-all" />
+              <menuitem action="diagram-unselect-all" />
+              <separator />
+            </placeholder>
           </menu>
           <menu action="diagram">
             <placeholder name="secondary">
@@ -41,14 +43,24 @@ class DiagramTab(object):
           </menu>
         </menubar>
         <toolbar name='mainwindow-toolbar'>
-          <separator />
-          <toolitem action="diagram-zoom-in" />
-          <toolitem action="diagram-zoom-out" />
-          <toolitem action="diagram-zoom-100" />
+          <placeholder name="left">
+            <separator />
+            <toolitem action="diagram-zoom-in" />
+            <toolitem action="diagram-zoom-out" />
+            <toolitem action="diagram-zoom-100" />
+          </placeholder>
         </toolbar>
       </ui>
     """
 
+    VIEW_TARGET_STRING = 0
+    VIEW_TARGET_ELEMENT_ID = 1
+    VIEW_TARGET_TOOLBOX_ACTION = 2
+    VIEW_DND_TARGETS = [
+        ('gaphor/element-id', 0, VIEW_TARGET_ELEMENT_ID),
+        ('gaphor/toolbox-action', 0, VIEW_TARGET_TOOLBOX_ACTION)]
+
+
     def __init__(self, owning_window):
         self.diagram = None
         self.view = None
@@ -85,8 +97,11 @@ class DiagramTab(object):
         """
         assert self.diagram
 
-        view = DiagramView(diagram=self.diagram)
-        scrolled_window = gtk.ScrolledWindow(view.hadjustment, view.vadjustment)
+        view = GtkView(canvas=self.diagram.canvas)
+        view.drag_dest_set(gtk.DEST_DEFAULT_ALL, DiagramTab.VIEW_DND_TARGETS,
+                           gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK)
+
+        scrolled_window = gtk.ScrolledWindow()
         scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         scrolled_window.set_shadow_type(gtk.SHADOW_IN)
         scrolled_window.add(view)
@@ -101,7 +116,7 @@ class DiagramTab(object):
 
         self.widget = scrolled_window
         
-        self.toolbox = DiagramToolbox(view)
+        self.toolbox = DiagramToolbox(self.diagram, view)
         
         return scrolled_window
 
@@ -125,13 +140,8 @@ class DiagramTab(object):
         Tab is destroyed. Do the same thing that would
         be done if File->Close was pressed.
         """
-        # Set diagram to None, so all refrences to the diagram are destroyed.
-        #self.widget.destroy()
-
         self.owning_window.remove_tab(self)
         self.set_diagram(None)
-        # We need this to get the view deleted properly:
-        #del self.view.diagram
         Application.unregister_handler(self._on_element_delete)
         Application.unregister_handler(self._on_element_change)
         del self.view
@@ -220,15 +230,12 @@ class DiagramTab(object):
         edit stuff.
         """
         if view.is_focus():
-            if event.keyval == 0xFFFF and \
-                    (event.state == 0 or event.state & gtk.gdk.MOD2_MASK): # Delete
+            if event.keyval == gtk.keysyms.Delete and \
+                    (event.state == 0 or event.state & gtk.gdk.MOD2_MASK):
                 self.delete_selected_items()
-            elif event.keyval == 0xFF08 and \
-                    (event.state == 0 or event.state & gtk.gdk.MOD2_MASK): # Backspace
+            elif event.keyval == gtk.keysyms.BackSpace and \
+                    (event.state == 0 or event.state & gtk.gdk.MOD2_MASK):
                 self.delete_selected_items()
-            else:
-                print '%x' %event.keyval, event.state
-                #self.action_manager.execute('diagram-delete')
 
 
     def _on_view_selection_changed(self, view, selection_or_focus):
@@ -239,18 +246,22 @@ class DiagramTab(object):
         """
         Handle data dropped on the canvas.
         """
-        if data and data.format == 8 and info == DiagramView.TARGET_TOOLBOX_ACTION:
+        if data and data.format == 8 and info == DiagramTab.VIEW_TARGET_TOOLBOX_ACTION:
             tool = self.toolbox.get_tool(data.data)
-            tool.create_item(view, x, y)
+            tool.create_item((x, y))
             context.finish(True, False, time)
-        elif data and data.format == 8 and info == DiagramView.TARGET_ELEMENT_ID:
+        elif data and data.format == 8 and info == DiagramTab.VIEW_TARGET_ELEMENT_ID:
             #print 'drag_data_received:', data.data, info
-            element = self.element_factory.lookup(data.data)
+            n, p = data.data.split('#')
+            element = self.element_factory.lookup(n)
             assert element
 
             # TODO: use adapters to execute code below
 
-            item_class = get_diagram_item(type(element))
+            q = type(element)
+            if p:
+                q = q, p
+            item_class = get_diagram_item(q)
             if isinstance(element, UML.Diagram):
                 self.action_manager.execute('OpenModelElement')
             elif item_class:
@@ -271,3 +282,4 @@ class DiagramTab(object):
         else:
             context.finish(False, False, time)
 
+# vim: sw=4:et:ai
