@@ -14,7 +14,6 @@ from namespace import NamespaceModel, NamespaceView
 from diagramtab import DiagramTab
 from toolbox import Toolbox
 from diagramtoolbox import TOOLBOX_ACTIONS
-from propertyeditor import PropertyEditor
 from toplevelwindow import ToplevelWindow
 
 
@@ -91,6 +90,9 @@ class MainWindow(ToplevelWindow):
           </menu>
         </menubar>
         <toolbar name='mainwindow-toolbar'>
+            <placeholder name="left" />
+            <separator expand="true" />
+            <placeholder name="right" />
         </toolbar>
         <toolbar action="tools">
         </toolbar>
@@ -135,12 +137,14 @@ class MainWindow(ToplevelWindow):
 
     tree_view = property(lambda s: s._tree_view)
 
+
     def get_filename(self):
         """
         Return the file name of the currently opened model.
         """
         return self.file_manager.filename
 
+
     def get_current_diagram_tab(self):
         """
         Get the currently opened and viewed DiagramTab, shown on the right
@@ -149,6 +153,7 @@ class MainWindow(ToplevelWindow):
         """
         return self.get_current_tab()
 
+
     def get_current_diagram(self):
         """
         Return the Diagram associated with the viewed DiagramTab.
@@ -157,6 +162,7 @@ class MainWindow(ToplevelWindow):
         tab = self.get_current_diagram_tab()
         return tab and tab.get_diagram()
 
+
     def get_current_diagram_view(self):
         """
         Return the DiagramView associated with the viewed DiagramTab.
@@ -165,6 +171,7 @@ class MainWindow(ToplevelWindow):
         tab = self.get_current_diagram_tab()
         return tab and tab.get_view()
 
+
     def ask_to_close(self):
         """
         Ask user to close window.
@@ -177,6 +184,7 @@ class MainWindow(ToplevelWindow):
         dialog.destroy()
         return answer == gtk.RESPONSE_YES
 
+
     def show_diagram(self, diagram):
         """
         Show a Diagram element in a new tab.
@@ -196,6 +204,7 @@ class MainWindow(ToplevelWindow):
 
         return tab
 
+
     def ui_component(self):
         """
         Create the widgets that make up the main window.
@@ -228,25 +237,11 @@ class MainWindow(ToplevelWindow):
         notebook.connect_after('switch-page', self._on_notebook_switch_page)
         notebook.connect_after('page-removed', self._on_notebook_page_removed)
 
-        self.property_editor = PropertyEditor()
-        pe_notebook = self.property_editor.construct()
-        pe_notebook.set_size_request(-1, 50)
-
-        second_paned = gtk.VPaned()
-        second_paned.set_property('position',
-                                 int(self.properties.get('ui.property-editor-position', 600)))
-        second_paned.pack1(notebook)
-        notebook.show()
-        second_paned.pack2(pe_notebook)
-        pe_notebook.show()
         
-        paned.pack2(second_paned)
-        second_paned.show()
+        paned.pack2(notebook)
+        notebook.show()
         paned.show()
 
-        second_paned.connect('notify::position',
-                            self._on_property_editor_notify_position)
-
         self.notebook = notebook
         self._tree_view = view
        
@@ -256,8 +251,11 @@ class MainWindow(ToplevelWindow):
 
         self._toolbox = toolbox
 
+        self.open_welcome_page()
+
         return paned
 
+
     def construct(self):
         super(MainWindow, self).construct()
 
@@ -271,18 +269,14 @@ class MainWindow(ToplevelWindow):
         Application.register_handler(self._action_executed)
         Application.register_handler(self._new_model_content)
 
-    def _update_toolbox(self, action_group):
+
+    def open_welcome_page(self):
         """
-        Update the buttons in the toolbox. Each button should be connected
-        by an action. Each button is assigned a special _action_name_
-        attribute that can be used to fetch the action from the ui manager.
+        Create a new tab with a textual welcome page, a sort of 101 for
+        Gaphor.
         """
-        for button in self._toolbox.buttons:
-            
-            action_name = button.action_name
-            action = action_group.get_action(action_name)
-            if action:
-                action.connect_proxy(button)
+        pass
+
 
     # Notebook methods:
 
@@ -296,6 +290,7 @@ class MainWindow(ToplevelWindow):
         l = gtk.Label(label)
         # Note: append_page() emits switch-page event
         self.notebook.append_page(contents, l)
+        self.notebook.set_tab_reorderable(contents, True)
         page_num = self.notebook.page_num(contents)
         #self.notebook.set_current_page(page_num)
         return page_num
@@ -484,34 +479,50 @@ class MainWindow(ToplevelWindow):
         # Make sure everyone knows the selection has changed.
         Application.handle(DiagramSelectionChange(tab.view, tab.view.focused_item, tab.view.selected_items))
 
+
     def _on_window_size_allocate(self, window, allocation):
         """
         Store the window size in a property.
         """
         self.properties.set('ui.window-size', (allocation.width, allocation.height))
 
-    def _on_property_editor_notify_position(self, paned, arg):
-        self.properties.set('ui.property-editor-position',
-                     paned.get_position())
+
+    def _update_toolbox(self, action_group):
+        """
+        Update the buttons in the toolbox. Each button should be connected
+        by an action. Each button is assigned a special _action_name_
+        attribute that can be used to fetch the action from the ui manager.
+        """
+        for button in self._toolbox.buttons:
+            
+            action_name = button.action_name
+            action = action_group.get_action(action_name)
+            if action:
+                action.connect_proxy(button)
+
 
     # Actions:
 
     @action(name='file-quit', stock_id='gtk-quit')
     def quit(self):
+        # TODO: check for changes (e.g. undo manager), fault-save
         self.ask_to_close() and gtk.main_quit()
         self._tree_view.get_model().close()
         Application.unregister_handler(self._action_executed)
         Application.unregister_handler(self._new_model_content)
 
+
     @action(name='tree-view-open', label='_Open')
     def tree_view_open_selected(self):
         element = self._tree_view.get_selected_element()
+        # TODO: Candidate for adapter?
         if isinstance(element, UML.Diagram):
             self.show_diagram(element)
         else:
             log.debug('No action defined for element %s' % type(element).__name__)
 
-    @action(name='tree-view-rename', label=_('Rename'))
+
+    @action(name='tree-view-rename', label=_('Rename'), accel='F2')
     def tree_view_rename_selected(self):
         view = self._tree_view
         element = view.get_selected_element()
@@ -523,6 +534,7 @@ class MainWindow(ToplevelWindow):
         view.set_cursor(path, column, True)
         cell.set_property('editable', 0)
 
+
     @action(name='tree-view-create-diagram', label=_('_New diagram'), stock_id='gaphor-diagram')
     @transactional
     def tree_view_create_diagram(self):
@@ -539,6 +551,7 @@ class MainWindow(ToplevelWindow):
         self.show_diagram(diagram)
         self.tree_view_rename_selected()
 
+
     @action(name='tree-view-delete-diagram', label=_('_Delete diagram'), stock_id='gtk-delete')
     @transactional
     def tree_view_delete_diagram(self):
@@ -550,7 +563,7 @@ class MainWindow(ToplevelWindow):
                               'This will possibly delete diagram items\n'
                               'that are not shown in other diagrams.'
                               % (diagram.name or '<None>'))
-        if (m.run() == gtk.RESPONSE_YES):
+        if m.run() == gtk.RESPONSE_YES:
             for i in reversed(diagram.canvas.get_all_items()):
                 s = i.subject
                 if s and len(s.presentation) == 1:
@@ -559,6 +572,7 @@ class MainWindow(ToplevelWindow):
             diagram.unlink()
         m.destroy()
 
+
     @action(name='tree-view-create-package', label=_('New _package'), stock_id='gaphor-package')
     @transactional
     def tree_view_create_package(self):
@@ -574,6 +588,7 @@ class MainWindow(ToplevelWindow):
         self.select_element(package)
         self.tree_view_rename_selected()
 
+
     @action(name='tree-view-delete-package', label=_('Delete pac_kage'), stock_id='gtk-delete')
     @transactional
     def tree_view_delete_package(self):
@@ -581,6 +596,7 @@ class MainWindow(ToplevelWindow):
         assert isinstance(package, UML.Package)
         package.unlink()
 
+
     @action(name='tree-view-refresh', label=_('_Refresh'))
     def tree_view_refresh(self):
         self._tree_view.get_model().refresh()
@@ -590,6 +606,7 @@ class MainWindow(ToplevelWindow):
     def reset_tool_after_create(self, active):
         self.properties.set('reset-tool-after-create', active)
 
+
 gtk.accel_map_add_filter('gaphor')
 
 
