@@ -13,19 +13,19 @@ from zope import component
 
 from gaphor.application import Application
 from gaphor import UML
-from gaphor.UML.event import ElementCreateEvent, ModelFactoryEvent, FlushFactoryEvent
+from gaphor.UML.event import ElementCreateEvent, ModelFactoryEvent, FlushFactoryEvent, DerivedSetEvent
 from gaphor.UML.interfaces import IAttributeChangeEvent, IElementDeleteEvent
-from gaphor.UML.event import DerivedUnionSetEvent
 from gaphor.transaction import Transaction
+from iconoption import get_icon_option
 
-
-# The following items will not be shown in the treeview, although they
+# The following items will be shown in the treeview, although they
 # are UML.Namespace elements.
 _default_filter_list = (
     UML.Class,
     UML.Interface,
     UML.Package,
     UML.Component,
+    UML.Device,
     UML.Node,
     UML.Artifact,
     UML.Interaction,
@@ -178,6 +178,8 @@ class NamespaceModel(gtk.GenericTreeModel):
         """
         if type(element) not in self.filter:
             return
+        if type(element) is UML.Property and element.namespace is None:
+            return
         if element.namespace and type(element.namespace) not in self.filter:
             return
 
@@ -191,7 +193,10 @@ class NamespaceModel(gtk.GenericTreeModel):
         # Add children
         if isinstance(element, UML.Namespace):
             for e in element.ownedMember:
-                self._add_elements(e)
+                # check if owned member is indeed within parent's namespace
+                # the check is important in case on Node classes
+                if element is e.namespace:
+                    self._add_elements(e)
 
 
     def _remove_element(self, element):
@@ -221,6 +226,8 @@ class NamespaceModel(gtk.GenericTreeModel):
     @catchall
     def _on_element_delete(self, event):
         element = event.element
+        if type(element) is UML.Property and element.namespace is None:
+            return
         if event.service is self.factory and \
                 type(element) in self.filter:
             path = self.path_from_element(element)
@@ -240,7 +247,7 @@ class NamespaceModel(gtk.GenericTreeModel):
 #                self.row_has_child_toggled(path[:-1], self.get_iter(path[:-1]))
 
 
-    @component.adapter(DerivedUnionSetEvent)
+    @component.adapter(DerivedSetEvent)
     @catchall
     def _on_association_set(self, event):
 
@@ -491,15 +498,21 @@ class NamespaceView(gtk.TreeView):
 
     def _set_pixbuf(self, column, cell, model, iter, data):
         value = model.get_value(iter, 0)
+        q = t = type(value)
+
+        p = get_icon_option(value)
+        if p is not None:
+            q = (t, p)
+
         try:
-            icon = self.icon_cache[type(value)]
+            icon = self.icon_cache[q]
         except KeyError:
-            stock_id = stock.get_stock_id(type(value))
+            stock_id = stock.get_stock_id(t, p)
             if stock_id:
                 icon = self.render_icon(stock_id, gtk.ICON_SIZE_MENU, '')
             else:
                 icon = None
-            self.icon_cache[type(value)] = icon
+            self.icon_cache[q] = icon
         cell.set_property('pixbuf', icon)
 
 
@@ -550,10 +563,13 @@ class NamespaceView(gtk.TreeView):
         model, iter = selection.get_selected()
         if iter:
             element = model.get_value(iter, 0)
+            p = get_icon_option(element)
+            p = p if p else ''
+            # 'id#stereotype' is being send
             if info == NamespaceView.TARGET_ELEMENT_ID:
-                selection_data.set(selection_data.target, 8, str(element.id))
+                selection_data.set(selection_data.target, 8, '%s#%s' % (element.id, p))
             else:
-                selection_data.set(selection_data.target, 8, element.name)
+                selection_data.set(selection_data.target, 8, '%s#%s' % (element.name, p))
 
 
     def on_drag_data_delete (self, context):
@@ -570,12 +586,12 @@ class NamespaceView(gtk.TreeView):
         """
         self.emit_stop_by_name('drag-data-received')
         #print 'drag_data_received'
-        data = selection.data
+        n, p = selection.data.split('#')
         drop_info = self.get_dest_row_at_pos(x, y)
         if drop_info:
             #print 'drop_info', drop_info
             model = self.get_model()
-            element = self.factory.lookup(data)
+            element = self.factory.lookup(n)
             path, position = drop_info
             iter = model.get_iter(path)
             dest_element = model.get_value(iter, 0)
